import * as React from 'react';
import * as ReactDOM from 'react-dom';

import {
  applyMiddleware,
  compose,
  createStore,
  GenericStoreEnhancer,
  Middleware,
  Store,
} from 'redux';

import {
  Provider,
} from 'react-redux';

import {
  ConnectedRouter,
  routerMiddleware,
} from 'react-router-redux';

import {
  createHashHistory,
  History,
} from 'history';

import {
  AppContainer,
} from 'react-hot-loader';

import { Application } from '@overflow/commons/application';
import {
  Runner,
  WebApplication,
} from '@overflow/commons/application/decorators';
import {
  Inject,
  Resource,
  Value,
} from '@overflow/commons/di/decorators';

import config, {
  injectables,
  jsonSources,
  ReducerConfiguration,
  RestAPIConfiguration,
  RpcAPIConfiguration,
  WebAppConfiguration,
} from './config';
import {WebAppDispatcherReducer} from './redux';
import WebApp from './pages/webapp';

// declare let module: { hot: any };

declare global {
  interface Window {
    devToolsExtension: () => any;
  }
  const process: any;
  const module: { hot: any };
}

class A {
}
class B extends A {
}

@WebApplication({
  injectables: injectables,
  jsonSources: jsonSources,
  configurations: [
    ReducerConfiguration,
    RestAPIConfiguration,
    RpcAPIConfiguration,
    WebAppConfiguration,
  ],
})
class WebAppApplication extends B {
  private static isProduction:boolean = process.env.NODE_ENV === 'production' ? true : false;
  private static useReduxDevTools:boolean = window.devToolsExtension && !WebAppApplication.isProduction ? true : false;

  @Resource()
  private containerPromise: Promise<HTMLElement>;
  private container: HTMLElement;

  @Resource()
  private store: Store<any>;
  @Resource()
  private history: History;

  @Inject()
  private dispatcherReducer: WebAppDispatcherReducer;

  @Runner()
  public async run(): Promise<void> {
    try {
      this.container = await this.containerPromise;
      this.renderLoading();
      let reducer: WebAppDispatcherReducer = new WebAppDispatcherReducer();
      reducer.registerReducers(config.redux.reducers);
      this.store = createStore(reducer.reducer, config.redux.state);

      this.renderApp();
    } catch (e) {
      console.error(e);
    }
  }

  private renderLoading(): void {
    ReactDOM.render(
        <div style={{
          width: '100vw',
          height: '100vh',
          backgroundColor: 'white',
          display: 'flex',
          alignItems: 'center',
          justifyContent: 'center',
          }}>
            <h1>Loading...</h1>
        </div>,
        this.container,
    );
  }

  private renderError(e: Error): void {
    ReactDOM.render(
        <div style={{
          width: '100vw',
          height: '100vh',
          backgroundColor: 'white',
          display: 'flex',
          alignItems: 'center',
          justifyContent: 'center',
          }}>
            <h1>{e.message}</h1>
        </div>,
        this.container,
    );
  }

  private renderApp(): void {
    WebAppApplication.isProduction ? this.renderProduction() : this.rederDevelopment();
    // ReactDOM.render(
    //   <Provider store={this.store}>
    //     <ConnectedRouter history={this.history}>
    //       <WebApp/>
    //     </ConnectedRouter>
    //   </Provider>
    //   ,
    //   this.container,
    // );

  }
  private renderProduction(): void {
    ReactDOM.render(
      <Provider store={this.store}>
        <ConnectedRouter history={this.history}>

        </ConnectedRouter>
      </Provider>,
      this.container,
    );
  }

  private rederDevelopment(): void {
    if (module.hot) {
      module.hot.accept('./pages/webapp', async () => {
        const NextApp = (await import('./pages/webapp')).default;
        ReactDOM.render(
          <AppContainer>
            <Provider store={this.store}>
              <ConnectedRouter history={this.history}>
                <NextApp />
              </ConnectedRouter>
            </Provider>
          </AppContainer>
          ,
          this.container,
        );
      });
    }

    ReactDOM.render(
      <AppContainer>
        <Provider store={this.store}>
          <ConnectedRouter history={this.history}>
            <WebApp/>
          </ConnectedRouter>
        </Provider>
      </AppContainer>
      ,
      this.container,
    );


  }



  public static main(): void {
    let webapp = new WebAppApplication();
    webapp.run();
  }
}

Application.run(WebAppApplication);
